/*
 * CLCKDEFS.H
 * This file contains constants and defines for
 * the hardware time tracker.
 *
 *
 * This is a public released file for specific use
 * with tinyUSBboard.
 *
 *
 * This is version 20150725T0730ZSB
 *
 * Stephan Baerwolf (matrixstorm@gmx.de), Schwansee 2015
 * for http://matrixstorm.com/avr/tinyusbboard/
 * (please contact me at least before commercial use)
 */


#ifndef __CLCKDEFS_H_f5122a1865724f109796ae8983840757
#define __CLCKDEFS_H_f5122a1865724f109796ae8983840757 1

#include "iocomfort.h"

#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>
#include <avr/io.h>

typedef struct __hwclock_time_t hwclock_time_t;

#if (defined(__AVR_ATmega8__)   || defined(__AVR_ATmega8A__)   ||    \
     defined(__AVR_ATmega88__)  || defined(__AVR_ATmega88P__)  ||    \
     defined(__AVR_ATmega168__) || defined(__AVR_ATmega168P__) ||    \
     defined(__AVR_ATmega328__) || defined(__AVR_ATmega328P__))
  // original supported avrs - layout 1:
  // connect PB1 (OC1B) with PD4 (T0)
#	define __HWCLOCK_layout			(1)
#	ifndef __HWCLOCK_PWM
#		define __HWCLOCK_PWM		B,2
#	endif
#	ifndef __HWCLOCK_TRIGGER
#		define __HWCLOCK_TRIGGER	D,4
#	endif
  // on these devices only 12MHz and 16MHz are supported
#	if (F_CPU == 12000000LL)
#		define __HWCLOCK_T0prescaler	(_BV(CS02) | _BV(CS01))				/* falling edge -- when timer 1 overflows  */
#		define __HWCLOCK_T1prescaler	(0*_BV(CS12) | 1*_BV(CS11) | 1*_BV(CS10))	/* prescaler 64: 5.333us per tick, 42sec total */
#		define __HWCLOCK_NSPERTICK	(5333ULL)
#	elif (F_CPU == 16000000LL)
#		define __HWCLOCK_T0prescaler	(_BV(CS02) | _BV(CS01))				/* falling edge -- when timer 1 overflows  */
#		define __HWCLOCK_T1prescaler	(0*_BV(CS12) | 1*_BV(CS11) | 1*_BV(CS10))	/* prescaler 64: 4us per tick, 33sec total */
#		define __HWCLOCK_NSPERTICK	(4000ULL)
#	else
#		error Unsupported clock frequency
#	endif

/* other than layout 1 */
#elif (defined(__AVR_ATmega32__)   || defined(__AVR_ATmega32A__)   ||    \
       defined(__AVR_ATmega644__)  || defined(__AVR_ATmega644P__)  ||    \
       defined(__AVR_ATmega1284__) || defined(__AVR_ATmega1284P__))
  // secondary support (layout based on ATmega32) - give an untested warning here
#	warning secondary AVR support not tested, yet
#	define __HWCLOCK_layout		(2)
  // connect PD4 (OC1B) with PB0 (T0)
#	ifndef __HWCLOCK_PWM
#		define __HWCLOCK_PWM		D,4
#	endif
#	ifndef __HWCLOCK_TRIGGER
#		define __HWCLOCK_TRIGGER	B,0
#	endif
  // clock frequency for secondary devices: 12MHz. 16MHz, 18MHz and 20MHz
#	if (F_CPU == 12000000LL)
#		define __HWCLOCK_T0prescaler	(_BV(CS02) | _BV(CS01))				/* falling edge -- when timer 1 overflows  */
#		define __HWCLOCK_T1prescaler	(0*_BV(CS12) | 1*_BV(CS11) | 1*_BV(CS10))	/* prescaler 64: 5.333us per tick, 42sec total */
#		define __HWCLOCK_NSPERTICK	(5333ULL)
#	elif (F_CPU == 16000000LL)
#		define __HWCLOCK_T0prescaler	(_BV(CS02) | _BV(CS01))				/* falling edge -- when timer 1 overflows  */
#		define __HWCLOCK_T1prescaler	(0*_BV(CS12) | 1*_BV(CS11) | 1*_BV(CS10))	/* prescaler 64: 4us per tick, 33sec total */
#		define __HWCLOCK_NSPERTICK	(4000ULL)
#	elif (F_CPU == 18000000LL)
#		define __HWCLOCK_T0prescaler	(_BV(CS02) | _BV(CS01))				/* falling edge -- when timer 1 overflows  */
#		define __HWCLOCK_T1prescaler	(0*_BV(CS12) | 1*_BV(CS11) | 1*_BV(CS10))	/* prescaler 64: about 5.555us per tick, 29.8sec total */
#		define __HWCLOCK_NSPERTICK	(3555ULL)
#	elif (F_CPU == 20000000LL)
#		define __HWCLOCK_T0prescaler	(_BV(CS02) | _BV(CS01))				/* falling edge -- when timer 1 overflows  */
#		define __HWCLOCK_T1prescaler	(0*_BV(CS12) | 1*_BV(CS11) | 1*_BV(CS10))	/* prescaler 64: about 3.2us per tick, 26.8sec total */
#		define __HWCLOCK_NSPERTICK	(3200ULL)
#	else
#		error Unsupported clock frequency
#	endif

/* no suitable layout */
#else
#	error AVR architecture not supported
#endif

#define __HWCLOCK_CEILDIV(x,y)	(((x)%(y))?(((x)/(y))+1):((x)/(y)))

#define HWCLOCK_UStoTICK(x)	(__HWCLOCK_CEILDIV((x*1000ULL), __HWCLOCK_NSPERTICK))
#define HWCLOCK_TICKtoUS(x)	(__HWCLOCK_CEILDIV((x*__HWCLOCK_NSPERTICK), 1000ULL))


#ifdef __HWCLOCK_C_f5122a1865724f109796ae8983840757
#	define	HWCLOCKPUBLIC
#else
#	define	HWCLOCKPUBLIC		extern
#endif




/* 
 * Gets the current time in an atomic fashion.
 * When returning the function result time may be in the recent history.
 */
HWCLOCKPUBLIC hwclock_time_t hwclock_now(void);

/*
 * Calculates the time differential (as a new time) of two input times.
 */
HWCLOCKPUBLIC hwclock_time_t hwclock_delta(const hwclock_time_t earlier, hwclock_time_t later);

/*
 * Calculates the time differential in ticks of two input times.
 */
HWCLOCKPUBLIC uint32_t hwclock_tickspassed(const hwclock_time_t earlier, hwclock_time_t later);

/*
 * Check a given timedelta, if it has passed a given time.
 * This function is faster than checks using "hwclock_tickspassed".
 */
HWCLOCKPUBLIC bool hwclock_ispassed(const hwclock_time_t timedelta, uint16_t passticks);

/* in case spinlocks are needed */
typedef int8_t (*hwclock_waitcallback_t)(const hwclock_time_t the_now, const hwclock_time_t waiting_since, const hwclock_time_t the_delta, const uint16_t waiting_ticks, void* userparameter);
HWCLOCKPUBLIC int8_t hwclock_spinwait(const uint16_t ticks, hwclock_waitcallback_t waitcallback, void* userparameter);

#endif
