/*
 * CORE.C
 * 
 * This file can be released as single C source,
 * simply containing a mapping and bare arduino
 * functions.
 * 
 * This is a public released file for specific use
 * with tinyUSBboard.
 *
 *
 * This is version 20150725T0730ZSB
 *
 * Stephan Baerwolf (matrixstorm@gmx.de), Schwansee 2015
 * for http://matrixstorm.com/avr/tinyusbboard/
 * (please contact me at least before commercial use)
 */

#define __CORE_C_ad1682ae837d4689b1f6aa371d65fb0a 1

#include "core.h"

const HIDMAPSECTION hidmap_mapentry_onstore_t hidmap_maptable_default[] = {
    // entry point, defining active chain via "nextchain" and init internal descriptors for the first time - chain 8 is the initial chain: mousemove
    {{0, SNES_KEYS_NONE()        , HIDMAP_PRESSTIME_ASAP   }, {8, hidmap_maprule_initboth, {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    
    // using the joypad mapping to cursors - select switches back to mousemode (via edge detect in chain 9)
    {{6, SNES_KEYS1(SNES_Select) , HWCLOCK_UStoTICK(100000)}, {9, hidmap_maprule_mapboth , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{6, SNES_KEYS1(SNES_R)      , HIDMAP_PRESSTIME_DEFAULT}, {6, hidmap_maprule_TXkey   , {{0                                , { 0,  0,  0}}}, {_BV(HIDKEYBOARD_MODBIT_LEFT_SHIFT) , HIDKEYBOARD_KEYUSE_delete      }}},
    {{6, SNES_KEYS1(SNES_L)      , HIDMAP_PRESSTIME_DEFAULT}, {6, hidmap_maprule_TXkey   , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDKEYBOARD_KEYUSE_delete      }}},
    {{6, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {6, hidmap_maprule_initkeys, {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{6, SNES_KEYS1(SNES_Up)     , HIDMAP_PRESSTIME_DEFAULT}, {6, hidmap_maprule_mapkey  , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDKEYBOARD_KEYUSE_arrow_up    }}},
    {{6, SNES_KEYS1(SNES_Down)   , HIDMAP_PRESSTIME_DEFAULT}, {6, hidmap_maprule_mapkey  , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDKEYBOARD_KEYUSE_arrow_down  }}},
    {{6, SNES_KEYS1(SNES_Left)   , HIDMAP_PRESSTIME_DEFAULT}, {6, hidmap_maprule_mapkey  , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDKEYBOARD_KEYUSE_arrow_left  }}},
    {{6, SNES_KEYS1(SNES_Right)  , HIDMAP_PRESSTIME_DEFAULT}, {6, hidmap_maprule_mapkey  , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDKEYBOARD_KEYUSE_arrow_right }}},
    {{6, SNES_KEYS1(SNES_Start)  , HIDMAP_PRESSTIME_DEFAULT}, {6, hidmap_maprule_mapkey  , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDKEYBOARD_KEYUSE_return      }}},
    {{6, SNES_KEYS1(SNES_A)      , HIDMAP_PRESSTIME_DEFAULT}, {6, hidmap_maprule_mapkey  , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDKEYBOARD_KEYUSE_a           }}},
    {{6, SNES_KEYS1(SNES_B)      , HIDMAP_PRESSTIME_DEFAULT}, {6, hidmap_maprule_mapkey  , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDKEYBOARD_KEYUSE_b           }}},
    {{6, SNES_KEYS1(SNES_Y)      , HIDMAP_PRESSTIME_DEFAULT}, {6, hidmap_maprule_mapkey  , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDKEYBOARD_KEYUSE_y           }}},
    {{6, SNES_KEYS1(SNES_X)      , HIDMAP_PRESSTIME_DEFAULT}, {6, hidmap_maprule_mapkey  , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDKEYBOARD_KEYUSE_x           }}},
    {{6, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {6, hidmap_maprule_cmtkey  , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{6, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {6, hidmap_maprule_respawn , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},

    // transition back to default chain (from 6 to 8, staying in 9 as long as select keeps beeing pressed (of course clean report buffers to avoid infinit mousemove or keypress)
    {{9, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {9, hidmap_maprule_initboth, {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{9, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {9, hidmap_maprule_cmtboth , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{9, SNES_KEYS1(SNES_Select) , HIDMAP_PRESSTIME_ASAP   }, {9, hidmap_maprule_respawn , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{9, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {8, hidmap_maprule_respawn , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},



    // transition to another chain (from 8 to 6, staying in 7 as long as select keeps beeing pressed (of course clean report buffers to avoid infinit mousemove or keypress)
    {{8, SNES_KEYS1(SNES_Select) , HWCLOCK_UStoTICK(100000)}, {7, hidmap_maprule_mapboth , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{7, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {7, hidmap_maprule_initboth, {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{7, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {7, hidmap_maprule_cmtboth , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{7, SNES_KEYS1(SNES_Select) , HIDMAP_PRESSTIME_ASAP   }, {7, hidmap_maprule_respawn , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{7, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {6, hidmap_maprule_respawn , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},

    // using the mouse means I can have different directions at the same time:
    {{8, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {8, hidmap_maprule_initmous, {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{8, SNES_KEYS1(SNES_Up)     , HIDMAP_PRESSTIME_DEFAULT}, {8, hidmap_maprule_mapmous , {{0                                , { 0, -9,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{8, SNES_KEYS1(SNES_Down)   , HIDMAP_PRESSTIME_DEFAULT}, {8, hidmap_maprule_mapmous , {{0                                , { 0, +9,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{8, SNES_KEYS1(SNES_Left)   , HIDMAP_PRESSTIME_DEFAULT}, {8, hidmap_maprule_mapmous , {{0                                , {-9,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{8, SNES_KEYS1(SNES_Right)  , HIDMAP_PRESSTIME_DEFAULT}, {8, hidmap_maprule_mapmous , {{0                                , {+9,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{8, SNES_KEYS1(SNES_R)      , HIDMAP_PRESSTIME_DEFAULT}, {8, hidmap_maprule_mapmous , {{0                                , { 0,  0, +1}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{8, SNES_KEYS1(SNES_L)      , HIDMAP_PRESSTIME_DEFAULT}, {8, hidmap_maprule_mapmous , {{0                                , { 0,  0, -1}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    // either leftklick
    {{8, SNES_KEYS1(SNES_Y)      , HIDMAP_PRESSTIME_DEFAULT}, {8, hidmap_maprule_cmtmous , {{_BV(mouse_report_buttonbit_left) , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{8, SNES_KEYS1(SNES_Y)      , HIDMAP_PRESSTIME_DEFAULT}, {8, hidmap_maprule_respawn , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    // or rightklick
    {{8, SNES_KEYS1(SNES_B)      , HIDMAP_PRESSTIME_DEFAULT}, {8, hidmap_maprule_cmtmous , {{_BV(mouse_report_buttonbit_right), { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{8, SNES_KEYS1(SNES_B)      , HIDMAP_PRESSTIME_DEFAULT}, {8, hidmap_maprule_respawn , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{8, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {8, hidmap_maprule_cmtmous , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    
    // if not respawned due to left- or rightlick yet: check some keys directly
    {{8, SNES_KEYS1(SNES_X)      , HIDMAP_PRESSTIME_DEFAULT}, {8, hidmap_maprule_TXkey   , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDKEYBOARD_KEYUSE_x           }}},
    {{8, SNES_KEYS1(SNES_A)      , HIDMAP_PRESSTIME_DEFAULT}, {8, hidmap_maprule_TXkey   , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDKEYBOARD_KEYUSE_a           }}},
    
    // rule, whose filter is always matching (be carefule, even if "SNES_KEYS_ALWAYS()" is set: Without "HIDMAP_PRESSTIME_ASAP" the rule still might not match if not some time has passed!
    {{8, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {8, hidmap_maprule_TXkey   , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
    {{8, SNES_KEYS_ALWAYS()      , HIDMAP_PRESSTIME_ASAP   }, {8, hidmap_maprule_respawn , {{0                                , { 0,  0,  0}}}, {0x00                               , HIDMAP_NOKEY                   }}},
};



void loop(void) {
  main_poll();
}
void setup(void) {
  main_initialization();
}


/* not in standard arduino available - but in case you want to
 * rollback some changes before entering bootloader
 */
// void conclude(void) {
//   main_finalization();
// }
