/*
 * MAPDEFS.H
 * This file contains public defines for the hidmap
 * mapping engine.
 * 
 * This is a public released file for specific use
 * with tinyUSBboard.
 *
 * This is version 20150725T0730ZSB
 *
 * Stephan Baerwolf (matrixstorm@gmx.de), Schwansee 2015
 * for http://matrixstorm.com/avr/tinyusbboard/
 * (please contact me at least before commercial use)
 */

#ifndef __MAPDEFS_H_70acd57d377f4c0fac4c68a786fc87ef
#define __MAPDEFS_H_70acd57d377f4c0fac4c68a786fc87ef 1

#include "asciimap.h"
#include "hidcore.h"
#include "snesdefs.h"
#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>
#include <avr/pgmspace.h>

/* allow an extra section of memory to implement the mapping */
#ifndef HIDMAPSECTION
#	ifdef DATASECTION
#		define HIDMAPSECTION DATASECTION
#	else
#		define HIDMAPSECTION PROGMEM
#	endif
#endif

#define HIDMAP_NOKEY			(HIDKEYBOARD_BOOT_KEYUSAGE_NoEvent)
#define HIDMAP_PRESSTIME_ASAP		(0x0)
#define HIDMAP_PRESSTIME_DEFAULT	(HWCLOCK_UStoTICK(SNESPAD_PAUSETIME_US+(30)))




typedef uint8_t                    hidmap_chainid_t;
typedef uint16_t                   hidmap_presstime_t;
typedef struct __hidmap_mapentry_onstore_t hidmap_mapentry_onstore_t;
typedef struct __hidmap_mapentry_t hidmap_mapentry_t;
typedef struct __hidmap_mapstate_t hidmap_mapstate_t;

/* callbacks need to do a "_next" or a "_reset" */
typedef void (*hidmap_callback_t)(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);

struct __hidmap_struct_filter {
  hidmap_chainid_t	chainid;
  snespad_keystate_t	sneskeys;
  hidmap_presstime_t	presstime_ticks;
} __attribute__((packed));

union __hidmap_union_mousereport {
  struct {
    uint8_t		padding1;
    int8_t		padding3[3];
  } __attribute__((packed));
  mouse_report_t	mousereport;
} __attribute__((packed));

struct __hidmap_struct_kbrpt_shrt {
  uint8_t				modifier;
  uint8_t				singlekeycode;
} __attribute__((packed));
struct __hidmap_struct_storerule_shrt {
    hidmap_chainid_t			nextchain;
    hidmap_callback_t			commitcallfunc;
    union __hidmap_union_mousereport	mouseunion;
    struct __hidmap_struct_kbrpt_shrt	keyboardreport;
} __attribute__((packed));

struct __hidmap_mapentry_onstore_t {
  struct __hidmap_struct_filter		filter;
  struct __hidmap_struct_storerule_shrt rule;
} __attribute__((packed));

#ifdef __HIDMAP_C_70acd57d377f4c0fac4c68a786fc87ef
#	define	HIDMAPPUBLIC
#else
#	define	HIDMAPPUBLIC		extern
HIDMAPPUBLIC const HIDMAPSECTION hidmap_mapentry_onstore_t hidmap_maptable_default[];
#endif




/* This callback function resets the current rule processing to start over checking all rules again */
#define hidmap_commit_none	hidmap_maprule_respawn
HIDMAPPUBLIC void hidmap_maprule_respawn(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);
/* all the other rules continue processing */

/* Init callback overwrite the HID reports, but don't mark them dirty. Neither the respawn - the next rule in the table is selected for processing */
#define hidmap_init_keys	hidmap_maprule_initkeys
#define hidmap_init_mouse	hidmap_maprule_initmous
#define hidmap_init_both	hidmap_maprule_initboth
HIDMAPPUBLIC void hidmap_maprule_initkeys(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);
HIDMAPPUBLIC void hidmap_maprule_initmous(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);
HIDMAPPUBLIC void hidmap_maprule_initboth(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);

/* Merge callbacks merge current changes to the reports and mark them active. Like "init" they don't respawn. */
#define hidmap_merge_keys	hidmap_maprule_mapkey
#define hidmap_merge_mous	hidmap_maprule_mapmous
#define hidmap_merge_both	hidmap_maprule_mapboth
HIDMAPPUBLIC void hidmap_maprule_mapkey	(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);
HIDMAPPUBLIC void hidmap_maprule_mapmous	(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);
HIDMAPPUBLIC void hidmap_maprule_mapboth	(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);

/* When finished a descriptor, merge it a last time and commit it to the HID */
#define hidmap_commit_keys	hidmap_maprule_cmtkey
#define hidmap_commit_mous	hidmap_maprule_cmtmous
#define hidmap_commit_both	hidmap_maprule_cmtboth
HIDMAPPUBLIC void hidmap_maprule_cmtkey	(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);
HIDMAPPUBLIC void hidmap_maprule_cmtmous	(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);
HIDMAPPUBLIC void hidmap_maprule_cmtboth	(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);


/* TRUE DIRECT MAPPING: When it is only neccessary to directly map, do an "init, merge, commit and respawn" in one rule */
HIDMAPPUBLIC void hidmap_maprule_TXkey	(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);
HIDMAPPUBLIC void hidmap_maprule_TXmous	(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);
HIDMAPPUBLIC void hidmap_maprule_TXboth	(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry, hidmap_mapstate_t *state, void* userparameter);



// /* clears a hidmap_mapentry_t by setting it to its initial default values */
HIDMAPPUBLIC void hidmap_mapentry_clear(hidmap_mapentry_t *mapentry);

/* loads the contents of a hidmap_mapentry_t from its specific pointer in its data section. */
HIDMAPPUBLIC void hidmap_mapentry_load(const uint_farptr_t tableentry_ptr, hidmap_mapentry_t *mapentry);

/* clears a hidmap_mapstate_t by setting it to its initial default values */
HIDMAPPUBLIC void hidmap_mapstate_clear(hidmap_mapstate_t *state);
HIDMAPPUBLIC void hidmap_mapstate_init(hidmap_mapstate_t *state, const uint_farptr_t tablebaseentry_ptr);

/* resets the scanning to the top of the ruletable */
HIDMAPPUBLIC void hidmap_mapstate_reset(hidmap_mapstate_t *state);

/* goes to next rule to scan  */
HIDMAPPUBLIC void hidmap_mapstate_next(hidmap_mapstate_t *state);

#endif
