/*
 * SNESDEFS.H
 * This file contains constants and defines for
 * the SNES Controller Gamepad.
 * 
 * It also defines some MACROS for iocomfort 
 * integrations.
 *
 * This is a public released file for specific use
 * with tinyUSBboard.
 *
 *
 * This is version 20150725T0730ZSB
 *
 * Stephan Baerwolf (matrixstorm@gmx.de), Schwansee 2015
 * for http://matrixstorm.com/avr/tinyusbboard/
 * (please contact me at least before commercial use)
 */


#ifndef __SNESDEFS_H_d3278dd9d06a48f1b808738e3270a9be
#define __SNESDEFS_H_d3278dd9d06a48f1b808738e3270a9be 1

#include "iocomfort.h"
#include <stdlib.h>
#include <stdbool.h>
#include <stdint.h>

#ifdef __SNESPAD_C_d3278dd9d06a48f1b808738e3270a9be
#	define	SNESPADPUBLIC
#else
#	define	SNESPADPUBLIC		extern
#endif


/* IOCOMFORT extension */
#define PINSNESKEY	snespad_currentkeystate
#define	SNES_B		SNESKEY,0
#define	SNES_Y		SNESKEY,1
#define	SNES_Select	SNESKEY,2
#define	SNES_Start	SNESKEY,3
#define	SNES_Up		SNESKEY,4
#define	SNES_Down	SNESKEY,5
#define	SNES_Left	SNESKEY,6
#define	SNES_Right	SNESKEY,7
#define	SNES_A		SNESKEY,8
#define	SNES_X		SNESKEY,9
#define	SNES_L		SNESKEY,10
#define	SNES_R		SNESKEY,11
#define	SNES_XtraButton	SNESKEY,12


#define SNESPAD_PAUSETIME_US					(16670)

#define SNES_KEYS1(keyA)					(_BV(_internal_BIT_OF(keyA)))
#define SNES_KEYS2(keyA, keyB)					(_BV(_internal_BIT_OF(keyA)) | _BV(_internal_BIT_OF(keyB)))
#define SNES_KEYS3(keyA, keyB, keyC)				(_BV(_internal_BIT_OF(keyA)) | _BV(_internal_BIT_OF(keyB)) | _BV(_internal_BIT_OF(keyC)))
#define SNES_KEYS4(keyA, keyB, keyC, keyD)			(_BV(_internal_BIT_OF(keyA)) | _BV(_internal_BIT_OF(keyB)) | _BV(_internal_BIT_OF(keyC)) | _BV(_internal_BIT_OF(keyD)))
#define SNES_KEYS5(keyA, keyB, keyC, keyD, keyE)		(_BV(_internal_BIT_OF(keyA)) | _BV(_internal_BIT_OF(keyB)) | _BV(_internal_BIT_OF(keyC)) | _BV(_internal_BIT_OF(keyD)) | _BV(_internal_BIT_OF(keyE)))
#define SNES_KEYS6(keyA, keyB, keyC, keyD, keyE, keyF)		(_BV(_internal_BIT_OF(keyA)) | _BV(_internal_BIT_OF(keyB)) | _BV(_internal_BIT_OF(keyC)) | _BV(_internal_BIT_OF(keyD)) | _BV(_internal_BIT_OF(keyE)) | _BV(_internal_BIT_OF(keyF)))

#if (__FEATURE_PROGBUTTONEXTRASNES > 0)
#	define SNES_KEYS_ALL(x)					((0xfff) | (_BV(BIT_OF(SNES_XtraButton))))
#else
#	define SNES_KEYS_ALL(x)					(0xfff)
#endif
#define		SNES_KEYS_ANY(x)				(0x7fff)
#define		SNES_KEYS_NONE(x)				(0xffff)
#define		SNES_KEYS_ALWAYS(x)				(0x0)
#define		SNES_KEYS_NEVER(x)				(0x8000)


#define SNES_IS_PRESSED_Ex(keystate, keys)			((keys==SNES_KEYS_ANY())?(snespad_keypressed(keystate)):((keys==SNES_KEYS_NONE())?(!(snespad_keypressed(keystate))):(!(keystate & keys))))


typedef uint16_t snespad_keystate_t;

SNESPADPUBLIC snespad_keystate_t snespad_lastkeystate;
SNESPADPUBLIC snespad_keystate_t snespad_currentkeystate;

/* 
 * Polls the gamepad for new data. (NON blocking)
 * It returns the current keypresses or "0" if current communication didn't got data.
 * If you need to read the recent keystate, please use "snespad_currentkeystate".
 * This function updates "snespad_currentkeystate" when return value > 0.
 */
SNESPADPUBLIC snespad_keystate_t snespad_poll(void);

/* Checks if any key is pressed. Returns true if it is so. */
SNESPADPUBLIC bool snespad_keypressed(const snespad_keystate_t keystate);

#endif
